@@ -26,6 +26,8 @@
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #include <gconf/gconf-client.h>
 
+#include "sound.h"
+
 extern int LINES;
 extern int COLUMNS;
 
@@ -85,7 +87,6 @@ private:
 	GtkWidget * w;
 
         GList * themeList;
-        GList * bgThemeList;
         
 	char *blockPixmap;
 	char *bgPixmap;
@@ -100,10 +101,20 @@ private:
 	bool paused;
 	int timeoutId;
 	bool onePause;
+	
+	bool inPlay;
 
 	void generateTimer(int level);
-	
-	static gint eventHandler(GtkWidget *widget, GdkEvent *event, void *d);
+
+	static gint keyPressHandler(GtkWidget *widget, GdkEvent *event, Tetris *t);
+	static gint keyReleaseHandler(GtkWidget *widget, GdkEvent *event, Tetris *t);
+	static gchar *decodeDropData(gchar * data, gint type);
+	void saveBgOptions();
+	static void decodeColour (guint16 *data, Tetris *t);
+	static void resetColour (Tetris *t);
+	static void dragDrop(GtkWidget *widget, GdkDragContext *context,
+			     gint x, gint y, GtkSelectionData *data, 
+			     guint info, guint time, Tetris *t);
 	static int timeoutHandler(void *d);
 	static int gameQuit(GtkWidget *widget, void *d);
 	static int gameNew(GtkWidget *widget, void *d);
@@ -114,6 +125,7 @@ private:
 	static int gameProperties(GtkWidget *widget, void *d);
 	static void setupdialogDestroy(GtkWidget *widget, void *d);
 	static void setupdialogResponse(GtkWidget *dialog, gint response_id, void *d);
+	static void setSound (GtkWidget *widget, gpointer data);
  	static void setRotateCounterClockWise(GtkWidget *widget, void *d);
 	static void setSelectionPreview(GtkWidget *widget, void *d);
 	static void setSelectionBlocks(GtkWidget *widget, void *d);
@@ -125,6 +137,9 @@ private:
 	static void startingLevelChanged (GtkWidget *spin, gpointer data);
 	
 	static void gconfNotify (GConfClient *tmp_client, guint cnx_id, GConfEntry *tmp_entry, gpointer tmp_data);
+	static gchar *gconfGetString (GConfClient *client, const char *key, const char *default_val);
+	static int gconfGetInt (GConfClient *client, const char *key, int default_val);
+	static gboolean gconfGetBoolean (GConfClient *client, const char *key, gboolean default_val);
 	void initOptions ();
 	void setOptions ();
 	void writeOptions ();
@@ -132,6 +147,9 @@ private:
 	
 	GdkPixbuf *image;
 	GdkPixbuf *bgimage;
+	gboolean usebg;
+
+	GdkColor bgcolour;
 
 	void fillMenu(GtkWidget *menu, char *pixname, char *dirname, GList ** list, bool addnone = false);
 	
@@ -148,6 +166,14 @@ private:
 	GtkWidget *do_preview_toggle;
 	GtkWidget *random_block_colors_toggle;
 	GtkWidget *rotate_counter_clock_wise_toggle;
+	GtkWidget *sound_toggle;
+	
+	int moveLeft;
+	int moveRight;
+	int moveDown;
+	int moveDrop;
+	int moveRotate;
+	int movePause;
 	
 	GnomeUIInfo *gameMenuPtr; 
 	GnomeUIInfo *gameSettingsPtr; 
